
require("config")

local function message_all_players(message)
    for _, player in pairs(game.players) do
		if player.valid and player.connected then
			player.print(message)
		end
	end
end

-- For unlocking recipes based on already researched techs:
local function update_recipes()

    local tech_table = 
    {
        {
            name = "compact-inserters",
            recipes = 
            {
                "fast-long-handed-inserter",
            },
        },
        {
            name = "compact-repair-packs",
            recipes = 
            {
                "advanced-repair-pack",
            },
        },
        {
            name = "compact-repair-packs-2",
            recipes = 
            {
                "ultimate-repair-pack",
            },
        },
        {
            name = "compact-roboports",
            recipes = 
            {
                "roboport-mk2",
            },
        },
        {
            name = "compact-roboports-2",
            recipes = 
            {
                "roboport-mk3",
            },
        },
    }
    
    -- Compact solar energy:
    for level = 1, solar_accumulator_tiers - 1 do
        table.insert(tech_table, 
        {
            name = level == 1 and "compact-solar-energy" or "compact-solar-energy-" .. level,
            recipes = 
            {
                "solar-panel-mk" .. level + 1,
                "accumulator-mk" .. level + 1,
                level == 1 and "substation-mk2" or nil,
            },
        })
    end
    
    -- Compact steam energy:
    for level = 1, boiler_engine_tiers - 1 do
        table.insert(tech_table, 
        {
            name = level == 1 and "compact-steam-energy" or "compact-steam-energy-" .. level,
            recipes = 
            {
                "boiler-mk" .. level + 1,
                "steam-engine-mk" .. level + 1,
            },
        })
    end
    
    -- Compact material processing:
    for level = 1, miner_furnace_tiers - 1 do
        table.insert(tech_table, 
        {
            name = level == 1 and "compact-material-processing" or "compact-material-processing-" .. level,
            recipes = 
            {
                "electric-furnace-mk" .. level + 1,
                "mining-drill-mk" .. level + 1,
            },
        })
    end
    
    -- Compact oil processing:
    for level = 1, refinery_chemplant_tiers - 1 do
        table.insert(tech_table, 
        {
            name = level == 1 and "compact-oil-processing" or "compact-oil-processing-" .. level,
            recipes = 
            {
                "refinery-mk" .. level + 1,
                "chemical-plant-mk" .. level + 1,
            },
        })
    end
    
    -- Compact fluid handling:
    for level = 1, pipe_storage_tank_tiers - 1 do
        table.insert(tech_table, 
        {
            name = level == 1 and "compact-fluid-handling" or "compact-fluid-handling-" .. level,
            recipes = 
            {
                "pipe-mk" .. level + 1,
                "pipe-to-ground-mk" .. level + 1,
                "storage-tank-mk" .. level + 1,
                "small-pump-mk" .. level + 1,
                "offshore-pump-mk" .. level + 1,
            },
        })
    end
    
    for _, force in pairs(game.forces) do
        force.reset_recipes()
        force.reset_technologies()
        
        -- create tech/recipe table once
        local techs = force.technologies
        local recipes = force.recipes
        
        for _, entry in ipairs(tech_table) do
            if techs[entry.name] then
                if techs[entry.name].researched then
                    for k, v in ipairs(entry.recipes) do
                        if recipes[v] then
                            recipes[v].enabled = true
                        end
                    end
                end
            end
        end
    end
    
    if not config.recipe_auto_update then
        message_all_players("Compact Factory recipes has been updated !")
    end
    
end

-- Runs when the mod is changed/updated:
script.on_configuration_changed(function(data)
    if data and data.mod_changes and data.mod_changes["Compact_Factory"] then
        update_recipes()
    end
end)

-- When you want the recipes to auto_update when loading a game:
script.on_load(function()
    if config.recipe_auto_update then update_recipes() end
end)

-- For manual recipe updating:
remote.add_interface("Compact_Factory", 
    {
        force_recipe_update = function() update_recipes() end,
    }
)