
-- For merging tables:
local function merge(t1, t2)
    assert(t1, "Table merge failed: No table1 provided !")
    assert(t2, "Table merge failed: No table2 provided !")
    for k, v in pairs(t2) do
        if (type(v) == "table") and (type(t1[k] or false) == "table") then
            merge(t1[k], t2[k])
        else
            t1[k] = v
        end
    end
    return t1
end

-- based_on entry handling:
local function register_entry(entry)
    assert(entry, "Data extension failed: No entry provided !")
    if entry.based_on ~= nil then
        data:extend({
            merge(table.deepcopy(entry.based_on), entry)
        })
    else
        data:extend({
            entry,
        })
    end
end

-- Accepts single entry or a table of entries:
function data_extend(entry_list)
    if entry_list.name then         -- kind of sketchy
        -- Provided just one entry
        register_entry(entry_list)
    else
        -- Provided list of entries
        for _, entry in pairs(entry_list) do
            register_entry(entry)
        end
    end
end

require("config") -- Get and parse config

-- All modifiers (WARNING - mathematical formulas):
function get_modifiers_for_tier(tier)
    
    return
    {
        -- General modifiers:
        general_solar_accumulator_mod  = math.pow(solar_accumulator_compression_level,  tier - 1),
        general_miner_furnace_mod      = math.pow(miner_furnace_compression_level,      tier - 1),
        general_refinery_chemplant_mod = math.pow(refinery_chemplant_compression_level, tier - 1),
        general_pipe_storage_tank_mod  = math.pow(pipe_storage_tank_compression_level,  tier - 1),
        general_boiler_engine_mod      = math.pow(boiler_engine_compression_level,      tier - 1),
        general_roboport_mod           = math.pow(roboport_compression_level,           tier - 1),
        
        general_health_mod = tier,
        
        -- Crafting speed modifiers:
        short_crafting_speed = (tier + 2) / 6,                -- short crafting speed (0.5s)
        
        solar_accumulator_crafting_speed = 5 * tier + 5,
        
        miner_crafting_speed   = 2 * tier,
        furnace_crafting_speed = 5 * tier,
        
        refinery_crafting_speed     = 10 * tier + 10,
        chemplant_crafting_speed    = 5 * tier + 5,
        storage_tank_crafting_speed = 2 * tier + 1,
        small_pump_crafting_speed   = 2 * tier,
        roboport_crafting_speed     = 5 * tier + 10,
        
        -- Misc:
        miner_animation_speed_mod = (2 * tier + 1) / 3,
        furnace_module_slots      = tier <= 2 and 2 or 3,
        boiler_effectivity_mod    = (tier + 2) / 3,
    }
    
end

require("prototypes.solar-accumulator")                                         -- All Solar Panels and Accumulators
require("prototypes.miner-furnace")                                             -- All Electric Mining Drills and Electric Furnaces
require("prototypes.refinery-chemplant")                                        -- All Refineries and Chemical Plants
require("prototypes.pipeworks")                                                 -- All Pipes, Pipes-to-Ground, Storage Tanks etc.
require("prototypes.boiler-engine")                                             -- All Boilers and Steam Engines
require("prototypes.roboports")                                                 -- All Roboports

require("prototypes.substation-mk2")                                            -- Substation MK2
require("prototypes.inserters")                                                 -- All Inserters
require("prototypes.repair-packs")                                              -- All Repair Packs

require("prototypes.technology.compact-solar-energy")                           -- All Technologies
require("prototypes.technology.compact-steam-energy")
require("prototypes.technology.compact-material-processing")
require("prototypes.technology.compact-oil-processing")
require("prototypes.technology.compact-fluid-handling")
require("prototypes.technology.compact-inserters")
require("prototypes.technology.compact-repair-packs")
require("prototypes.technology.compact-roboports")
