local boiler_default_entity       = data.raw.boiler.boiler
local steam_engine_default_entity = data.raw.generator["steam-engine"]

local boiler_default_item       = data.raw.item.boiler
local steam_engine_default_item = data.raw.item["steam-engine"]

local boiler_base_health      = boiler_default_entity.max_health
local boiler_base_consumption = tonumber(string.match(boiler_default_entity.energy_consumption, '%d+'))
local boiler_base_effectivity = boiler_default_entity.burner.effectivity

local steam_engine_base_health      = steam_engine_default_entity.max_health
local steam_engine_base_effectivity = steam_engine_default_entity.effectivity
local steam_engine_base_fluid_usage = steam_engine_default_entity.fluid_usage_per_tick

local function pipepictures_for_tier(tier)
    
    local path  = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier
    
    return
    {
        straight_vertical_single     = { filename = path .. "/pipe-straight-vertical-single.png" },
        straight_vertical            = { filename = path .. "/pipe-straight-vertical.png" },
        straight_vertical_window     = { filename = path .. "/pipe-straight-vertical-window.png" },
        straight_horizontal_window   = { filename = path .. "/pipe-straight-horizontal-window.png" },
        straight_horizontal          = { filename = path .. "/pipe-straight-horizontal.png" },
        corner_up_right              = { filename = path .. "/pipe-corner-up-right.png" },
        corner_up_left               = { filename = path .. "/pipe-corner-up-left.png" },
        corner_down_right            = { filename = path .. "/pipe-corner-down-right.png" },
        corner_down_left             = { filename = path .. "/pipe-corner-down-left.png" },
        t_up                         = { filename = path .. "/pipe-t-up.png" },
        t_down                       = { filename = path .. "/pipe-t-down.png" },
        t_right                      = { filename = path .. "/pipe-t-right.png" },
        t_left                       = { filename = path .. "/pipe-t-left.png" },
        cross                        = { filename = path .. "/pipe-cross.png" },
        ending_up                    = { filename = path .. "/pipe-ending-up.png" },
        ending_down                  = { filename = path .. "/pipe-ending-down.png" },
        ending_right                 = { filename = path .. "/pipe-ending-right.png" },
        ending_left                  = { filename = path .. "/pipe-ending-left.png" },
        horizontal_window_background = { filename = path .. "/pipe-horizontal-window-background.png" },
        vertical_window_background   = { filename = path .. "/pipe-vertical-window-background.png" },
    }
end

local function pipecoverspictures_for_tier(tier)
    
    local path = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier
    
    return 
    {
        north = { filename = path .. "/pipe-cover-north.png" },
        east  = { filename = path .. "/pipe-cover-east.png" },
        south = { filename = path .. "/pipe-cover-south.png" },
        west  = { filename = path .. "/pipe-cover-west.png" },
    }
end

local function get_ingredients(name, tier)          -- name: "boiler" or "engine"
    
    local ingr = {}
    
    if name == "boiler" then
        if tier == 2 then
            table.insert(ingr, {"steel-plate", 10} )
            table.insert(ingr, {"stone-brick", 10} )
            table.insert(ingr, {"effectivity-module", 4} )
        elseif tier == 3 then
            table.insert(ingr, {"steel-plate", 15} )
            table.insert(ingr, {"plastic-bar", 10} )
            table.insert(ingr, {"effectivity-module-2", 2} )
        elseif tier >= 4 then
            table.insert(ingr, {"steel-plate", 20} )
            table.insert(ingr, {"effectivity-module-3", 1} )
            table.insert(ingr, {"alien-artifact", 5} )
        end
    elseif name == "engine" then
        if tier == 2 then
            table.insert(ingr, {"iron-gear-wheel", 20} )
            table.insert(ingr, {"steel-plate", 10} )
            table.insert(ingr, {"speed-module", 4} )
        elseif tier == 3 then
            table.insert(ingr, {"engine-unit", 10} )
            table.insert(ingr, {"advanced-circuit", 15} )
            table.insert(ingr, {"speed-module-2", 2} )
        elseif tier >= 4 then
            table.insert(ingr, {"steel-plate", 20} )
            table.insert(ingr, {"speed-module-3", 1} )
            table.insert(ingr, {"alien-artifact", 5} )
        end
    end
    
    if name == "boiler" then
        if tier == 2 then
            table.insert(ingr, 1, { boiler_default_item.name, boiler_engine_compression_level })
        else
            table.insert(ingr, 1, { "boiler-mk" .. tier - 1, boiler_engine_compression_level })
        end
    elseif name == "engine" then
        if tier == 2 then
            table.insert(ingr, 1, { steam_engine_default_item.name, boiler_engine_compression_level })
        else
            table.insert(ingr, 1, { "steam-engine-mk" .. tier - 1, boiler_engine_compression_level })
        end
    end
    
    return ingr
    
end

local function get_boiler_table_for_tier(tier)
    
    local boiler_path = "__Compact_Factory__/graphics/boilers/boiler_mk" .. tier
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "boiler",
            name = "boiler-mk" .. tier,
            icon = "__Compact_Factory__/graphics/boilers/boiler_mk" .. tier .. "/boiler-icon.png",
            based_on = boiler_default_entity,
            minable = { result = "boiler-mk" .. tier },
            max_health = boiler_base_health * mods.general_health_mod,
            fluid_box =
            {
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            energy_consumption = boiler_base_consumption * mods.general_boiler_engine_mod .. "kW",
            burner =
            {
                effectivity = boiler_base_effectivity * mods.boiler_effectivity_mod,
            },
            structure =
            {
                left       = { filename = boiler_path .. "/boiler-left.png" },
                down       = { filename = boiler_path .. "/boiler-down.png" },
                left_down  = { filename = boiler_path .. "/boiler-left-down.png" },
                right_down = { filename = boiler_path .. "/boiler-right-down.png" },
                left_up    = { filename = boiler_path .. "/boiler-left-up.png" },
                right_up   = { filename = boiler_path .. "/boiler-right-up.png" },
                t_down     = { filename = boiler_path .. "/boiler-t-down.png" },
                t_up       = { filename = boiler_path .. "/boiler-t-up.png" },
            },
            -- these are the pipe pictures - boiler is a pipe as well
            pictures = pipepictures_for_tier(tier),
            fast_replaceable_group = "pipe",
        },
        item = 
        {
            type = "item",
            name = "boiler-mk" .. tier,
            icon = boiler_path .. "/boiler-icon.png",
            based_on = boiler_default_item,
            subgroup = "energy",
            order = "b[steam-power]-a[boiler][" .. tier .. "]",
            place_result = "boiler-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "boiler-mk" .. tier,
            energy_required = mods.short_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("boiler", tier),
            result = "boiler-mk" .. tier,
        },
    }
    
end

local function get_engine_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "generator",
            name = "steam-engine-mk" .. tier,
            icon = "__Compact_Factory__/graphics/steam_engines/engine_mk" .. tier .. "/steam-engine-icon.png",
            based_on = steam_engine_default_entity,
            minable = { result = "steam-engine-mk" .. tier },
            max_health = steam_engine_base_health * mods.general_health_mod,
            effectivity = steam_engine_base_effectivity * math.sqrt(mods.general_boiler_engine_mod),
            fluid_usage_per_tick = steam_engine_base_fluid_usage * math.sqrt(mods.general_boiler_engine_mod),
            fluid_box =
            {
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            horizontal_animation = { filename = "__Compact_Factory__/graphics/steam_engines/engine_mk" .. tier .. "/steam-engine-horizontal.png" },
            vertical_animation   = { filename = "__Compact_Factory__/graphics/steam_engines/engine_mk" .. tier .. "/steam-engine-vertical.png" },
            fast_replaceable_group = "steam_engine",
        },
        item = 
        {
            type = "item",
            name = "steam-engine-mk" .. tier,
            icon = "__Compact_Factory__/graphics/steam_engines/engine_mk" .. tier .. "/steam-engine-icon.png",
            based_on = steam_engine_default_item,
            subgroup = "energy",
            order = "b[steam-power]-b[steam-engine][" .. tier .. "]",
            place_result = "steam-engine-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "steam-engine-mk" .. tier,
            enabled = false,
            energy_required = mods.short_crafting_speed,
            ingredients = get_ingredients("engine", tier),
            result = "steam-engine-mk" .. tier,
        },
    }
    
end

local Boilers = {}
local Steam_Engines = {}

for tier = 2 , boiler_engine_tiers do
    
    if config.boilers then
        local boilers = get_boiler_table_for_tier(tier)
        table.insert(Boilers, boilers.entity)
        table.insert(Boilers, boilers.item)
        table.insert(Boilers, boilers.recipe)
    end
    
    if config.steam_engines then
        local steam_engines = get_engine_table_for_tier(tier)
        table.insert(Steam_Engines, steam_engines.entity)
        table.insert(Steam_Engines, steam_engines.item)
        table.insert(Steam_Engines, steam_engines.recipe)
    end
    
end

data_extend(Boilers)
data_extend(Steam_Engines)

-- Make original entities fast replacable with their higher tier counterparts:
steam_engine_default_entity.fast_replaceable_group = "steam_engine"
