
local miner_default_entity   = data.raw["mining-drill"]["electric-mining-drill"]
local furnace_default_entity = data.raw.furnace["electric-furnace"]

local miner_default_item   = data.raw.item["electric-mining-drill"]
local furnace_default_item = data.raw.item["electric-furnace"]

local miner_base_health = miner_default_entity.max_health
local miner_base_speed  = miner_default_entity.mining_speed
local miner_base_energy = tonumber(string.match(miner_default_entity.energy_usage, '%d+'))
local miner_base_animation_speed = miner_default_entity.animations.north.animation_speed

local furnace_base_health = furnace_default_entity.max_health
local furnace_base_speed  = furnace_default_entity.crafting_speed
local furnace_base_energy = tonumber(string.match(furnace_default_entity.energy_usage, '%d+'))

local function get_miner_animations(sprite_path, speed)
    
    return
    {
        north =
        {
            filename = sprite_path .. "/north.png",
            animation_speed = speed,
        },
        east =
        {
            filename = sprite_path .. "/east.png",
            animation_speed = speed,
        },
        south =
        {
            filename = sprite_path .. "/south.png",
            animation_speed = speed,
        },
        west =
        {
            filename = sprite_path .. "/west.png",
            animation_speed = speed,
        },
    }
    
end

local function get_ingredients(name, tier)          -- name: "miner" or "furnace"
    
    local ingr = {}
    
    if tier == 2 then
        ingr = 
        {
            {"advanced-circuit", 20},
            {"steel-plate", 20},
            {"speed-module", 4},
        }
    elseif tier == 3 then
        ingr = 
        {
            {"processing-unit", 20},
            {"speed-module-2", 2},
            {"effectivity-module-2", 2},
        }
    elseif tier >= 4 then
        ingr = 
        {
            {"alien-artifact", 10},
            {"speed-module-3", 1},
        }
    end
    
    if name == "miner" then
        if tier == 2 then
            table.insert(ingr, 1, { miner_default_item.name, miner_furnace_compression_level })
        else
            table.insert(ingr, 1, { "mining-drill-mk" .. tier - 1, miner_furnace_compression_level })
        end
    elseif name == "furnace" then
        if tier == 2 then
            table.insert(ingr, 1, { furnace_default_item.name, miner_furnace_compression_level })
        else
            table.insert(ingr, 1, { "electric-furnace-mk" .. tier - 1, miner_furnace_compression_level })
        end
    end
    
    return ingr
    
end

local function get_miner_table_for_tier(tier)

    mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "mining-drill",
            name = "mining-drill-mk" .. tier,
            icon = "__Compact_Factory__/graphics/mining_drills/drill_mk" .. tier .. "/mining-drill-icon.png",
            based_on = miner_default_entity,
            minable = { result = "mining-drill-mk" .. tier },
            max_health = miner_base_health * mods.general_health_mod,
            
            animations = get_miner_animations("__Compact_Factory__/graphics/mining_drills/drill_mk" .. tier, miner_base_animation_speed * mods.miner_animation_speed_mod),
            
            mining_speed = miner_base_speed  * mods.general_miner_furnace_mod,
            energy_usage = miner_base_energy * mods.general_miner_furnace_mod .. "kW",
            resource_searching_radius = tier <= 2 and 2.49 or 3.49,
        },
        item = 
        {
            type = "item",
            name = "mining-drill-mk" .. tier,
            icon = "__Compact_Factory__/graphics/mining_drills/drill_mk" .. tier .. "/mining-drill-icon.png",
            based_on = miner_default_item,
            order = "a[items]-b[mining-drill][".. tier .. "]",
            place_result = "mining-drill-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "mining-drill-mk" .. tier,
            energy_required = mods.miner_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("miner", tier),
            result = "mining-drill-mk" .. tier,
        },
    }
    
end

local function get_furnace_table_for_tier(tier)

    mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "furnace",
            name = "electric-furnace-mk" .. tier,
            icon = "__Compact_Factory__/graphics/electric_furnaces/furnace_mk" .. tier .. "/electric-furnace-icon.png",
            based_on = furnace_default_entity,
            minable = { result = "electric-furnace-mk" .. tier },
            max_health = furnace_base_health * mods.general_health_mod,
            module_specification =
            {
                module_slots = mods.furnace_module_slots,
            },
            animation =
            {
                filename = "__Compact_Factory__/graphics/electric_furnaces/furnace_mk" .. tier .. "/electric-furnace.png",
            },
            crafting_speed = furnace_base_speed * mods.general_miner_furnace_mod,
            energy_usage = furnace_base_energy * mods.general_miner_furnace_mod .. "kW",
        },
        item = 
        {
            type = "item",
            name = "electric-furnace-mk" .. tier,
            icon = "__Compact_Factory__/graphics/electric_furnaces/furnace_mk" .. tier .. "/electric-furnace-icon.png",
            based_on = furnace_default_item,
            order = "c[electric-furnace]-b[" .. tier .. "]",
            place_result = "electric-furnace-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "electric-furnace-mk" .. tier,
            energy_required = mods.furnace_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("furnace", tier),
            result = "electric-furnace-mk" .. tier,
        },
    }
    
end

local MiningDrills = {}
local ElectricFurnaces = {}

for tier = 2 , miner_furnace_tiers do
    
    if config.mining_drills then
        local miner = get_miner_table_for_tier(tier)
        table.insert(MiningDrills, miner.entity)
        table.insert(MiningDrills, miner.item)
        table.insert(MiningDrills, miner.recipe)
    end
    
    if config.electric_furnaces then
        local furnace = get_furnace_table_for_tier(tier)
        table.insert(ElectricFurnaces, furnace.entity)
        table.insert(ElectricFurnaces, furnace.item)
        table.insert(ElectricFurnaces, furnace.recipe)
    end
    
end

data_extend(MiningDrills)
data_extend(ElectricFurnaces)
