local pipe_default_entity          = data.raw.pipe["pipe"]
local ground_pipe_default_entity   = data.raw["pipe-to-ground"]["pipe-to-ground"]
local storage_tank_default_entity  = data.raw["storage-tank"]["storage-tank"]
local small_pump_default_entity    = data.raw.pump["small-pump"]
local offshore_pump_default_entity = data.raw["offshore-pump"]["offshore-pump"]

local pipe_default_item          = data.raw.item["pipe"]
local ground_pipe_default_item   = data.raw.item["pipe-to-ground"]
local storage_tank_default_item  = data.raw.item["storage-tank"]
local small_pump_default_item    = data.raw.item["small-pump"]
local offshore_pump_default_item = data.raw.item["offshore-pump"]

local pipe_base_health   = pipe_default_entity.max_health
local pipe_base_capacity = pipe_default_entity.fluid_box.base_area

local ground_pipe_base_health   = ground_pipe_default_entity.max_health
local ground_pipe_base_capacity = ground_pipe_default_entity.fluid_box.base_area

local storage_tank_base_health   = storage_tank_default_entity.max_health
local storage_tank_base_capacity = storage_tank_default_entity.fluid_box.base_area

local small_pump_base_health   = small_pump_default_entity.max_health
local small_pump_base_capacity = small_pump_default_entity.fluid_box.base_area
local small_pump_base_energy   = tonumber(string.match(small_pump_default_entity.energy_usage, '%d+'))
local small_pump_base_speed    = small_pump_default_entity.pumping_speed

local offshore_pump_base_health   = offshore_pump_default_entity.max_health
local offshore_pump_base_capacity = offshore_pump_default_entity.fluid_box.base_area
local offshore_pump_base_speed    = offshore_pump_default_entity.pumping_speed

local function pipepictures_for_tier(tier)
    
    local path  = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier
    
    return
    {
        straight_vertical_single     = { filename = path .. "/pipe-straight-vertical-single.png" },
        straight_vertical            = { filename = path .. "/pipe-straight-vertical.png" },
        straight_vertical_window     = { filename = path .. "/pipe-straight-vertical-window.png" },
        straight_horizontal_window   = { filename = path .. "/pipe-straight-horizontal-window.png" },
        straight_horizontal          = { filename = path .. "/pipe-straight-horizontal.png" },
        corner_up_right              = { filename = path .. "/pipe-corner-up-right.png" },
        corner_up_left               = { filename = path .. "/pipe-corner-up-left.png" },
        corner_down_right            = { filename = path .. "/pipe-corner-down-right.png" },
        corner_down_left             = { filename = path .. "/pipe-corner-down-left.png" },
        t_up                         = { filename = path .. "/pipe-t-up.png" },
        t_down                       = { filename = path .. "/pipe-t-down.png" },
        t_right                      = { filename = path .. "/pipe-t-right.png" },
        t_left                       = { filename = path .. "/pipe-t-left.png" },
        cross                        = { filename = path .. "/pipe-cross.png" },
        ending_up                    = { filename = path .. "/pipe-ending-up.png" },
        ending_down                  = { filename = path .. "/pipe-ending-down.png" },
        ending_right                 = { filename = path .. "/pipe-ending-right.png" },
        ending_left                  = { filename = path .. "/pipe-ending-left.png" },
        horizontal_window_background = { filename = path .. "/pipe-horizontal-window-background.png" },
        vertical_window_background   = { filename = path .. "/pipe-vertical-window-background.png" },
    }
end

local function pipecoverspictures_for_tier(tier)
    
    local path = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier
    
    return 
    {
        north = { filename = path .. "/pipe-cover-north.png" },
        east  = { filename = path .. "/pipe-cover-east.png" },
        south = { filename = path .. "/pipe-cover-south.png" },
        west  = { filename = path .. "/pipe-cover-west.png" },
    }
end

local function get_ingredients(name, tier)          -- name: "pipe" or "ground_pipe" or "storage_tank" or "small_pump" or "offshore_pump"
    
    local ingr = {}
    
    if name == "pipe" then
        if tier == 2 then
            table.insert(ingr, {"steel-plate", 2} )
        elseif tier == 3 then
            table.insert(ingr, {"steel-plate", 1} )
            table.insert(ingr, {"plastic-bar", 2} )
        elseif tier >= 4 then
            table.insert(ingr, {"stone-brick", 2} )
            table.insert(ingr, {"steel-plate", 2} )
        end
    elseif name == "ground_pipe" or name == "small_pump" then
        if tier == 2 then
            table.insert(ingr, {"steel-plate", 4} )
        elseif tier == 3 then
            table.insert(ingr, {"steel-plate", 2} )
            table.insert(ingr, {"plastic-bar", 4} )
        elseif tier >= 4 then
            table.insert(ingr, {"stone-brick", 2} )
            table.insert(ingr, {"steel-plate", 4} )
        end
    elseif name == "storage_tank" then
        if tier == 2 then
            table.insert(ingr, {"copper-plate", 20} )
            table.insert(ingr, {"steel-plate", 20} )
        elseif tier == 3 then
            table.insert(ingr, {"plastic-bar", 20} )
            table.insert(ingr, {"steel-plate", 20} )
        elseif tier >= 4 then
            table.insert(ingr, {"stone-brick", 20} )
            table.insert(ingr, {"steel-plate", 40} )
        end
    elseif name == "offshore_pump" then
        if tier == 2 then
            table.insert(ingr, {"iron-gear-wheel", 15} )
            table.insert(ingr, {"pipe", 10} )
            table.insert(ingr, {"steel-plate", 10} )
        elseif tier == 3 then
            table.insert(ingr, {"iron-gear-wheel", 15} )
            table.insert(ingr, {"speed-module", 2} )
            table.insert(ingr, {"steel-plate", 20} )
        elseif tier >= 4 then
            table.insert(ingr, {"engine-unit", 4} )
            table.insert(ingr, {"speed-module-2", 1} )
            table.insert(ingr, {"steel-plate", 30} )
        end
    end
    
    if name == "pipe" then
        if tier == 2 then
            table.insert(ingr, 1, { pipe_default_item.name, pipe_storage_tank_compression_level })
        else
            table.insert(ingr, 1, { "pipe-mk" .. tier - 1, pipe_storage_tank_compression_level })
        end
    elseif name == "ground_pipe" then
        if tier == 2 then
            table.insert(ingr, 1, { ground_pipe_default_item.name, pipe_storage_tank_compression_level * 2 })
        else
            table.insert(ingr, 1, { "pipe-to-ground-mk" .. tier - 1, pipe_storage_tank_compression_level * 2 })
        end
    elseif name == "storage_tank" then
        if tier == 2 then
            table.insert(ingr, 1, { storage_tank_default_item.name, pipe_storage_tank_compression_level })
        else
            table.insert(ingr, 1, { "storage-tank-mk" .. tier - 1, pipe_storage_tank_compression_level })
        end
    elseif name == "small_pump" then
        if tier == 2 then
            table.insert(ingr, 1, { small_pump_default_item.name, pipe_storage_tank_compression_level })
        else
            table.insert(ingr, 1, { "small-pump-mk" .. tier - 1, pipe_storage_tank_compression_level })
        end
    elseif name == "offshore_pump" then
        if tier == 2 then
            table.insert(ingr, 1, { offshore_pump_default_item.name, pipe_storage_tank_compression_level })
        else
            table.insert(ingr, 1, { "offshore-pump-mk" .. tier - 1, pipe_storage_tank_compression_level })
        end
    end
    
    return ingr
    
end

local function get_pipe_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "pipe",
            name = "pipe-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-icon.png",
            based_on = pipe_default_entity,
            minable = { result = "pipe-mk" .. tier },
            max_health = pipe_base_health * mods.general_health_mod,
            fluid_box =
            {
                base_area = pipe_base_capacity * math.sqrt(mods.general_pipe_storage_tank_mod),
            },
            pictures = pipepictures_for_tier(tier),
            fast_replaceable_group = "pipe",
        },
        item = 
        {
            type = "item",
            name = "pipe-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-icon.png",
            based_on = pipe_default_item,
            subgroup = "pipes",
            order = "a[pipe]-a[pipe][" .. tier .. "]",
            place_result = "pipe-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "pipe-mk" .. tier,
            energy_required = mods.short_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("pipe", tier),
            result = "pipe-mk" .. tier,
        },
    }
    
end

local function get_ground_pipe_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "pipe-to-ground",
            name = "pipe-to-ground-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-icon.png",
            based_on = ground_pipe_default_entity,
            minable = { result = "pipe-to-ground-mk" .. tier },
            max_health = ground_pipe_base_health * mods.general_health_mod,
            fluid_box =
            {
                base_area = ground_pipe_base_capacity * math.sqrt(mods.general_pipe_storage_tank_mod),
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            pictures =
            {
                up    = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-up.png" },
                down  = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-down.png" },
                left  = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-left.png" },
                right = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-right.png" },
            },
            fast_replaceable_group = "pipe_to_ground",
        },
        item = 
        {
            type = "item",
            name = "pipe-to-ground-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/pipe-to-ground-icon.png",
            based_on = ground_pipe_default_item,
            subgroup = "pipes",
            order = "a[pipe]-b[pipe-to-ground][" .. tier .. "]",
            place_result = "pipe-to-ground-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "pipe-to-ground-mk" .. tier,
            energy_required = mods.short_crafting_speed * 2,
            enabled = false,
            ingredients = get_ingredients("ground_pipe", tier),
            result = "pipe-to-ground-mk" .. tier,
            result_count = 2,
        },
    }
    
end

local function get_storage_tank_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "storage-tank",
            name = "storage-tank-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/storage-tank-icon.png",
            based_on = storage_tank_default_entity,
            minable = { result = "storage-tank-mk" .. tier },
            max_health = storage_tank_base_health * mods.general_health_mod,
            fluid_box =
            {
                base_area = storage_tank_base_capacity * mods.general_pipe_storage_tank_mod,
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            pictures =
            {
                picture =
                {
                    sheet = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/storage-tank.png" },
                },
            },
            fast_replaceable_group = "storage_tank",
        },
        item = 
        {
            type = "item",
            name = "storage-tank-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/storage-tank-icon.png",
            based_on = storage_tank_default_item,
            order = "b[fluid]-a[storage-tank][" .. tier .. "]",
            place_result = "storage-tank-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "storage-tank-mk" .. tier,
            energy_required = mods.storage_tank_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("storage_tank", tier),
            result = "storage-tank-mk" .. tier,
        },
    }
    
end

local function get_small_pump_table_for_tier(tier)
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "pump",
            name = "small-pump-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-icon.png",
            based_on = small_pump_default_entity,
            minable = { result = "small-pump-mk" .. tier },
            max_health = small_pump_base_health * mods.general_health_mod,
            fluid_box =
            {
                base_area = small_pump_base_capacity * mods.general_pipe_storage_tank_mod,
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            energy_usage = small_pump_base_energy * mods.general_pipe_storage_tank_mod .. "kW",
            pumping_speed = small_pump_base_speed * mods.general_pipe_storage_tank_mod,
            animations =
            {
                north = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-up.png" },
                east  = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-right.png" },
                south = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-down.png" },
                west  = { filename = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-left.png" },
            },
            fast_replaceable_group = "pipe",
        },
        item = 
        {
            type = "item",
            name = "small-pump-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/small-pump-icon.png",
            based_on = small_pump_default_item,
            order = "b[pipe]-c[small-pump][" .. tier .. "]",
            place_result = "small-pump-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "small-pump-mk" .. tier,
            energy_required = mods.small_pump_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("small_pump", tier),
            result = "small-pump-mk" .. tier,
        },
    }
end

local function get_offshore_pump_table_for_tier(tier)
    
    local offshore_pump_picture = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/offshore-pump.png"
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "offshore-pump",
            name = "offshore-pump-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/offshore-pump-icon.png",
            based_on = offshore_pump_default_entity,
            minable = { result = "offshore-pump-mk" .. tier },
            max_health = offshore_pump_base_health * mods.general_health_mod,
            fluid_box =
            {
                base_area = offshore_pump_base_capacity * mods.general_pipe_storage_tank_mod,
                pipe_covers = pipecoverspictures_for_tier(tier),
            },
            pumping_speed = offshore_pump_base_speed * mods.general_pipe_storage_tank_mod,
            picture =
            {
                north = { filename = offshore_pump_picture },
                east  = { filename = offshore_pump_picture },
                south = { filename = offshore_pump_picture },
                west  = { filename = offshore_pump_picture },
            },
        },
        item = 
        {
            type = "item",
            name = "offshore-pump-mk" .. tier,
            icon = "__Compact_Factory__/graphics/pipework/pipework_mk" .. tier .. "/offshore-pump-icon.png",
            based_on = offshore_pump_default_item,
            subgroup = "extraction-machine",
            order = "b[fluids]-a[offshore-pump][" .. tier .. "]",
            place_result = "offshore-pump-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "offshore-pump-mk" .. tier,
            energy_required = mods.short_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("offshore_pump", tier),
            result = "offshore-pump-mk" .. tier,
        },
    }
end

local Pipes = {}
local Pipes_To_Ground = {}
local Storage_Tanks = {}
local Small_Pumps = {}
local Offshore_Pumps = {}

for tier = 2 , pipe_storage_tank_tiers do
    
    if config.pipes then
        local pipes = get_pipe_table_for_tier(tier)
        table.insert(Pipes, pipes.entity)
        table.insert(Pipes, pipes.item)
        table.insert(Pipes, pipes.recipe)
        
        local ground_pipes = get_ground_pipe_table_for_tier(tier)
        table.insert(Pipes_To_Ground, ground_pipes.entity)
        table.insert(Pipes_To_Ground, ground_pipes.item)
        table.insert(Pipes_To_Ground, ground_pipes.recipe)
    end
    
    if config.storage_tanks then
        local storage_tanks = get_storage_tank_table_for_tier(tier)
        table.insert(Storage_Tanks, storage_tanks.entity)
        table.insert(Storage_Tanks, storage_tanks.item)
        table.insert(Storage_Tanks, storage_tanks.recipe)
    end
    
    if config.small_pumps then
        local small_pumps = get_small_pump_table_for_tier(tier)
        table.insert(Small_Pumps, small_pumps.entity)
        table.insert(Small_Pumps, small_pumps.item)
        table.insert(Small_Pumps, small_pumps.recipe)
    end
    
    if config.offshore_pumps then
        local offshore_pumps = get_offshore_pump_table_for_tier(tier)
        table.insert(Offshore_Pumps, offshore_pumps.entity)
        table.insert(Offshore_Pumps, offshore_pumps.item)
        table.insert(Offshore_Pumps, offshore_pumps.recipe)
    end
    
end

-- Add subgroups:
data:extend({
{
    type  = "item-subgroup",
    name  = "pipes",
    group = "logistics",
    order = "da",
},
})

data_extend(Pipes)
data_extend(Pipes_To_Ground)
data_extend(Storage_Tanks)
data_extend(Small_Pumps)
data_extend(Offshore_Pumps)

-- Change the original Small Pump graphic to make it more distinguishable from the MK4 Pump:
if config.small_pumps then
    small_pump_default_entity.icon = "__Compact_Factory__/graphics/pipework/small-pump-icon.png"
    small_pump_default_item.icon   = "__Compact_Factory__/graphics/pipework/small-pump-icon.png"
end

-- Move original crafting recipes/icons to proper subgroups:
if config.pipes then
    pipe_default_item.subgroup        = "pipes"
    ground_pipe_default_item.subgroup = "pipes"
end

-- Make original entities fast replacable with their higher tier counterparts:
ground_pipe_default_entity.fast_replaceable_group  = "pipe_to_ground"
storage_tank_default_entity.fast_replaceable_group = "storage_tank"
