
local refinery_default_entity  = data.raw["assembling-machine"]["oil-refinery"]
local chemplant_default_entity = data.raw["assembling-machine"]["chemical-plant"]

local refinery_default_item  = data.raw.item["oil-refinery"]
local chemplant_default_item = data.raw.item["chemical-plant"]

local refinery_base_health = refinery_default_entity.max_health
local refinery_base_speed  = refinery_default_entity.crafting_speed
local refinery_base_energy = tonumber(string.match(refinery_default_entity.energy_usage, '%d+'))

local chemplant_base_health = chemplant_default_entity.max_health
local chemplant_base_speed  = chemplant_default_entity.crafting_speed
local chemplant_base_energy = tonumber(string.match(chemplant_default_entity.energy_usage, '%d+'))

local function get_refinery_chemplant_animations(sprite_path)
    
    return
    {
        north = { filename = sprite_path },
        east  = { filename = sprite_path },
        south = { filename = sprite_path },
        west  = { filename = sprite_path },
    }
    
end

local function get_ingredients(name, tier)          -- name: "refinery" or "chemplant"
    
    local ingr = {}
    
    if tier == 2 then
        table.insert(ingr, {"advanced-circuit", 20} )
        table.insert(ingr, {"steel-plate", 20} )
        table.insert(ingr, {"speed-module", 4} )
    elseif tier == 3 then
        table.insert(ingr, {"processing-unit", 20} )
        table.insert(ingr, {"speed-module-2", 2} )
        table.insert(ingr, {"effectivity-module-2", 2} )
    elseif tier >= 4 then
        table.insert(ingr, {"alien-artifact", 10} )
        table.insert(ingr, {"speed-module-3", 1} )
    end
    
    if name == "refinery" then
        if tier == 2 then
            table.insert(ingr, 1, { refinery_default_item.name, refinery_chemplant_compression_level })
        else
            table.insert(ingr, 1, { "refinery-mk" .. tier - 1, refinery_chemplant_compression_level })
        end
    elseif name == "chemplant" then
        if tier == 2 then
            table.insert(ingr, 1, { chemplant_default_item.name, refinery_chemplant_compression_level })
        else
            table.insert(ingr, 1, { "chemical-plant-mk" .. tier - 1, refinery_chemplant_compression_level })
        end
    end
    
    return ingr
    
end

local function get_refinery_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "assembling-machine",
            name = "refinery-mk" .. tier,
            icon = "__Compact_Factory__/graphics/refineries/refinery_mk" .. tier .. "/refinery-icon.png",
            based_on = refinery_default_entity,
            minable = { result = "refinery-mk" .. tier },
            max_health = refinery_base_health * mods.general_health_mod,
            module_specification =
            {
                module_slots = 2,
            },
            crafting_speed = refinery_base_speed * mods.general_refinery_chemplant_mod,
            energy_usage  = refinery_base_energy * mods.general_refinery_chemplant_mod .. "kW",
            animation = get_refinery_chemplant_animations("__Compact_Factory__/graphics/refineries/refinery_mk" .. tier .. "/refinery.png"),
            fast_replaceable_group = "refinery",
        },
        item = 
        {
            type = "item",
            name = "refinery-mk" .. tier,
            icon = "__Compact_Factory__/graphics/refineries/refinery_mk" .. tier .. "/refinery-icon.png",
            based_on = refinery_default_item,
            subgroup = "oil-machine",
            order = "d[refinery][" .. tier .. "]",
            place_result = "refinery-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "refinery-mk" .. tier,
            energy_required = mods.refinery_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("refinery", tier),
            result = "refinery-mk" .. tier,
        },
    }
    
end

local function get_chemplant_table_for_tier(tier)

    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "assembling-machine",
            name = "chemical-plant-mk" .. tier,
            icon = "__Compact_Factory__/graphics/chemical_plants/chemical_plant_mk" .. tier .. "/chemical-plant-icon.png",
            based_on = chemplant_default_entity,
            minable = { result = "chemical-plant-mk" .. tier },
            max_health = chemplant_base_health * mods.general_health_mod,
            module_specification =
            {
                module_slots = 2,
            },
            animation = get_refinery_chemplant_animations("__Compact_Factory__/graphics/chemical_plants/chemical_plant_mk" .. tier .. "/chemical-plant.png"),
            crafting_speed = chemplant_base_speed * mods.general_refinery_chemplant_mod,
            energy_usage  = chemplant_base_energy * mods.general_refinery_chemplant_mod .. "kW",
            fast_replaceable_group = "chemical_plant",
        },
        item = 
        {
            type = "item",
            name = "chemical-plant-mk" .. tier,
            icon = "__Compact_Factory__/graphics/chemical_plants/chemical_plant_mk" .. tier .. "/chemical-plant-icon.png",
            based_on = chemplant_default_item,
            subgroup = "oil-machine",
            order = "e[chemical-plant][" .. tier .. "]",
            place_result = "chemical-plant-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "chemical-plant-mk" .. tier,
            energy_required = mods.chemplant_crafting_speed,
            enabled = false,
            ingredients = get_ingredients("chemplant", tier),
            result = "chemical-plant-mk" .. tier,
        },
    }
    
end

local Refineries = {}
local ChemicalPlants = {}

for tier = 2 , refinery_chemplant_tiers do
    
    if config.refineries then
        local refinery = get_refinery_table_for_tier(tier)
        table.insert(Refineries, refinery.entity)
        table.insert(Refineries, refinery.item)
        table.insert(Refineries, refinery.recipe)
    end
    
    if config.chemical_plants then
        local chemplant = get_chemplant_table_for_tier(tier)
        table.insert(ChemicalPlants, chemplant.entity)
        table.insert(ChemicalPlants, chemplant.item)
        table.insert(ChemicalPlants, chemplant.recipe)
    end
    
end

-- Add subgroups:
data:extend({
{
    type  = "item-subgroup",
    name  = "oil-machine",
    group = "production",
    order = "ea",
},
})

data_extend(Refineries)
data_extend(ChemicalPlants)

if config.refineries or config.chemical_plants then
    -- Move original crafting recipes/icons to proper subgroups:
    refinery_default_item.subgroup  = "oil-machine"
    chemplant_default_item.subgroup = "oil-machine"
end

-- Make original entities fast replacable with their higher tier counterparts:
refinery_default_entity.fast_replaceable_group  = "refinery"
chemplant_default_entity.fast_replaceable_group = "chemical_plant"
