local roboport_default_entity = data.raw.roboport.roboport
local roboport_default_item   = data.raw.item.roboport

local roboport_base_health           = roboport_default_entity.max_health
local roboport_base_energy_usage     = tonumber(string.match(roboport_default_entity.energy_usage, '%d+'))
local roboport_base_input_flow_limit = tonumber(string.match(roboport_default_entity.energy_source.input_flow_limit, '%d+'))

local function get_charging_offsets(tier)                                       -- the only thing that doesn't get calculated
    
    if tier == 1 then
        return
        {
            {-1.5, -0.5},
            {1.5, -0.5},
            {1.5, 1.5},
            {-1.5, 1.5},
        }
    elseif tier == 2 then
        return
        {
            {-1.25, -0.75}, {-1.75, -0.25},
            {1.25, -0.75},  {1.75, -0.25},
            {1.25, 1.75},   {1.75, 1.25},
            {-1.25, 1.75},  {-1.75, 1.25},
        }
    elseif tier >= 3 then
        return
        {
            {-1.25, -0.75}, {-1.75, -0.25}, {-1.75, -1.25}, {-2.25, -0.75},
            {1.25, -0.75},  {1.75, -0.25},  {1.75, -1.25},  {2.25, -0.75},
            {1.25, 1.75},   {1.75, 1.25},   {1.75, 2.25},   {2.25, 1.75},
            {-1.25, 1.75},  {-1.75, 1.25},  {-1.75, 2.25},  {-2.25, 1.75},
        }
    end
    
end

local function get_ingredients(tier)
    
    local ingr = {}
    
    if tier <= 2 then
        table.insert(ingr, {"steel-plate", 30} )
        table.insert(ingr, {"advanced-circuit", 20} )
    elseif tier >= 3 then
        table.insert(ingr, {"steel-plate", 40} )
        table.insert(ingr, {"processing-unit", 10} )
    end
    
    if tier == 2 then
        table.insert(ingr, 1, { roboport_default_item.name, roboport_compression_level })
    else
        table.insert(ingr, 1, { "roboport-mk" .. tier - 1, roboport_compression_level })
    end
    
    return ingr
    
end

local function get_roboport_table_for_tier(tier)
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "roboport",
            name = "roboport-mk" .. tier,
            icon = "__Compact_Factory__/graphics/roboports/roboport_mk" .. tier .. "/roboport-icon.png",
            based_on = roboport_default_entity,
            minable = { result = "roboport-mk" .. tier },
            max_health = roboport_base_health * mods.general_health_mod,
            energy_source =
            {
                input_flow_limit = roboport_base_input_flow_limit * mods.general_roboport_mod .. "MW",
            },
            energy_usage = roboport_base_energy_usage * mods.general_roboport_mod .. "kW",
            charging_offsets = get_charging_offsets(tier),
            base = { filename = "__Compact_Factory__/graphics/roboports/roboport_mk" .. tier .. "/roboport-base.png", },
            fast_replaceable_group = "roboport",
        },
        item = 
        {
            type = "item",
            name = "roboport-mk" .. tier,
            icon = "__Compact_Factory__/graphics/roboports/roboport_mk" .. tier .. "/roboport-icon.png",
            based_on = roboport_default_item,
            order = "c[signal]-a[roboport][" .. tier .. "]",
            place_result = "roboport-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "roboport-mk" .. tier,
            energy_required = mods.roboport_crafting_speed,
            enabled = false,
            ingredients = get_ingredients(tier),
            result = "roboport-mk" .. tier,
        },
    }
    
end

local Roboports = {}

for tier = 2 , roboports_tiers do
    
    if config.roboports then
        local roboports = get_roboport_table_for_tier(tier)
        table.insert(Roboports, roboports.entity)
        table.insert(Roboports, roboports.item)
        table.insert(Roboports, roboports.recipe)
    end
    
end

data_extend(Roboports)

-- Make original entities fast replacable with their higher tier counterparts:
roboport_default_entity.fast_replaceable_group = "roboport"
