
local solar_default_entity = data.raw["solar-panel"]["solar-panel"]
local accu_default_entity  = data.raw.accumulator["basic-accumulator"] or data.raw.accumulator["accumulator"]

local solar_default_item = data.raw.item["solar-panel"]
local accu_default_item  = data.raw.item["basic-accumulator"] or data.raw.item["accumulator"]

local solar_base_health = solar_default_entity.max_health
local solar_base_energy = tonumber(string.match(solar_default_entity.production, '%d+'))

local accumulator_base_health   = accu_default_entity.max_health
local accumulator_base_capacity = tonumber(string.match(accu_default_entity.energy_source.buffer_capacity, '%d+'))
local accumulator_base_input    = tonumber(string.match(accu_default_entity.energy_source.input_flow_limit, '%d+'))
local accumulator_base_output   = tonumber(string.match(accu_default_entity.energy_source.output_flow_limit, '%d+'))

local function get_ingredients(name, tier)          -- name: "solar" or "accu"
    
    local ingr = {}
    
    if tier == 2 then
        table.insert(ingr, {"electronic-circuit", 20})
        table.insert(ingr, {"steel-plate", 10})
    elseif tier == 3 then
        table.insert(ingr, {"advanced-circuit", 20})
        table.insert(ingr, {"steel-plate", 20})
    elseif tier == 4 then
        table.insert(ingr, {"processing-unit", 20})
        table.insert(ingr, {"steel-plate", 30})
    end
    
    if name == "solar" and tier >= 5 then
        table.insert(ingr, {"fusion-reactor-equipment", 1})
    elseif name == "accu" and tier >= 5 then
        table.insert(ingr, {"battery-mk2-equipment", 1})
        table.insert(ingr, {"alien-artifact", 30})
    end
    
    if name == "solar" then
        if tier == 2 then
            table.insert(ingr, 1, { solar_default_item.name, solar_accumulator_compression_level })
        else
            table.insert(ingr, 1, { "solar-panel-mk" .. tier - 1, solar_accumulator_compression_level })
        end
    elseif name == "accu" then
        if tier == 2 then
            table.insert(ingr, 1, { accu_default_item.name, solar_accumulator_compression_level })
        else
            table.insert(ingr, 1, { "accumulator-mk" .. tier - 1, solar_accumulator_compression_level })
        end
    end
    
    return ingr
    
end

local function get_solar_table_for_tier(tier)
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "solar-panel",
            name = "solar-panel-mk" .. tier,
            icon = "__Compact_Factory__/graphics/solar_panels/solar_mk" .. tier .. "/solar-panel-icon.png",
            based_on = solar_default_entity,
            minable = { result = "solar-panel-mk" .. tier },
            max_health = solar_base_health * mods.general_health_mod,
            picture =
            {
                filename = "__Compact_Factory__/graphics/solar_panels/solar_mk" .. tier .. "/solar-panel.png",
            },
            production = solar_base_energy * mods.general_solar_accumulator_mod .. "kW",
            fast_replaceable_group = "solar_panel",
        },
        item = 
        {
            type = "item",
            name = "solar-panel-mk" .. tier,
            icon = "__Compact_Factory__/graphics/solar_panels/solar_mk" .. tier .. "/solar-panel-icon.png",
            based_on = solar_default_item,
            subgroup = "solar_panels",
            order = "d[solar-panel]-b[" .. tier .. "]",
            place_result = "solar-panel-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "solar-panel-mk" .. tier,
            energy_required = mods.solar_accumulator_crafting_speed,
            enabled = "false",
            ingredients = get_ingredients("solar", tier),
            result = "solar-panel-mk" .. tier,
        },
    }
    
end

local function get_accumulator_table_for_tier(tier)
    
    local mods = get_modifiers_for_tier(tier)
    
    return
    {
        entity = 
        {
            type = "accumulator",
            name = "accumulator-mk" .. tier,
            icon = "__Compact_Factory__/graphics/accumulators/acc_mk" .. tier .. "/accumulator-icon.png",
            based_on = accu_default_entity,
            minable = { result = "accumulator-mk" .. tier },
            max_health = accumulator_base_health * mods.general_health_mod,
            energy_source =
            {
                type = "electric",
                buffer_capacity   = accumulator_base_capacity * mods.general_solar_accumulator_mod .. "MJ",
                input_flow_limit  = accumulator_base_input    * mods.general_solar_accumulator_mod .. "kW",
                output_flow_limit = accumulator_base_output   * mods.general_solar_accumulator_mod .. "kW",
            },
            picture =
            {
                filename = "__Compact_Factory__/graphics/accumulators/acc_mk" .. tier .. "/accumulator.png",
            },
            charge_animation =
            {
                filename = "__Compact_Factory__/graphics/accumulators/acc_mk" .. tier .. "/accumulator-charge-animation.png",
                animation_speed = 0.5,
            },
            discharge_animation =
            {
                filename = "__Compact_Factory__/graphics/accumulators/acc_mk" .. tier .. "/accumulator-discharge-animation.png",
                animation_speed = 0.5,
            },
            fast_replaceable_group = "accumulator",
        },
        item = 
        {
            type = "item",
            name = "accumulator-mk" .. tier,
            icon = "__Compact_Factory__/graphics/accumulators/acc_mk" .. tier .. "/accumulator-icon.png",
            based_on = accu_default_item,
            subgroup = "accumulators",
            order = "e[accumulator]-b[" .. tier .. "]",
            place_result = "accumulator-mk" .. tier,
        },
        recipe = 
        {
            type = "recipe",
            name = "accumulator-mk" .. tier,
            energy_required = mods.solar_accumulator_crafting_speed,
            enabled = "false",
            ingredients = get_ingredients("accu", tier),
            result = "accumulator-mk" .. tier,
        },
    }
    
end

local SolarPanels = {}
local Accumulators = {}

for tier = 2 , solar_accumulator_tiers do
    
    if config.solar_panels then
        local solar = get_solar_table_for_tier(tier)
        table.insert(SolarPanels, solar.entity)
        table.insert(SolarPanels, solar.item)
        table.insert(SolarPanels, solar.recipe)
    end
    
    if config.accumulators then
        local accu = get_accumulator_table_for_tier(tier)
        table.insert(Accumulators, accu.entity)
        table.insert(Accumulators, accu.item)
        table.insert(Accumulators, accu.recipe)
    end
    
end

-- Add subgroups:
data:extend({
{
    type  = "item-subgroup",
    name  = "solar_panels",
    group = "production",
    order = "ba",
},
{
    type  = "item-subgroup",
    name  = "accumulators",
    group = "production",
    order = "bb",
},
})

data_extend(SolarPanels)
data_extend(Accumulators)

-- Move original crafting recipes/icons to proper subgroups:
if config.solar_panels then
    solar_default_item.subgroup = "solar_panels"
end
if config.accumulators then
    accu_default_item.subgroup = "accumulators"
end

-- Make original entities fast replacable with their higher tier counterparts:
solar_default_entity.fast_replaceable_group = "solar_panel"
accu_default_entity.fast_replaceable_group  = "accumulator"
