
local function get_effects_for_level(level)
    
    local effects = {}
    
    if config.pipes then
        table.insert(effects, { type = "unlock-recipe", recipe = "pipe-mk" .. level + 1 })
        table.insert(effects, { type = "unlock-recipe", recipe = "pipe-to-ground-mk" .. level + 1 })
    end
    if config.storage_tanks then
        table.insert(effects, { type = "unlock-recipe", recipe = "storage-tank-mk" .. level + 1 })
    end
    if config.small_pumps then
        table.insert(effects, { type = "unlock-recipe", recipe = "small-pump-mk" .. level + 1 })
    end
    if config.offshore_pumps then
        table.insert(effects, { type = "unlock-recipe", recipe = "offshore-pump-mk" .. level + 1 })
    end
    
    return effects
    
end

local function get_ingredients_for_level(level)
    
    local blue = 
    {
        {"science-pack-1", 1},
        {"science-pack-2", 1},
        {"science-pack-3", 1},
    }
    
    if level <= 2 then
        return blue
    else
        table.insert(blue, {"alien-science-pack", 1})
        return blue
    end
    
end

local function get_prequisites_for_level(level)
    
    local preqs = {}
    
    if level == 1 then
        preqs = { "fluid-handling", "steel-processing" }
    elseif level == 2 then
        preqs = { "speed-module-2", "plastics" }
    elseif level == 3 then
        preqs = { "alien-technology" }
    end
    
    if level == 2 then
        table.insert(preqs, 1, "compact-fluid-handling" )
    elseif level >= 3 then
        table.insert(preqs, 1, "compact-fluid-handling-" .. level - 1 )
    end
    
    return preqs
    
end

local function make_compact_tech_for_level(level)
    
    return
    {
        type = "technology",
        name = level == 1 and "compact-fluid-handling" or "compact-fluid-handling-" .. level,
        icon = "__Compact_Factory__/graphics/technology/compact-fluid-handling-" .. level + 1 .. ".png",
        icon_size = 128,
        effects = get_effects_for_level(level),
        prerequisites = get_prequisites_for_level(level),
        upgrade = true,
        unit =
        {
            count = ( 100 * math.pow(2, level - 1) ) + 100,
            ingredients = get_ingredients_for_level(level),
            time = 15 * level,
        },
        order = "d-b-a",
    }
    
end

local CompactTech = {}

if config.pipes or config.storage_tanks or config.small_pumps or config.offshore_pumps or config.preserve_techs then
    for level = 1, pipe_storage_tank_tiers - 1 do
        table.insert(CompactTech, make_compact_tech_for_level(level))
    end
end

data_extend(CompactTech)
