
local function get_effects_for_level(level)
    
    local effects = {}
    
    if config.refineries then
        table.insert(effects, { type = "unlock-recipe", recipe = "refinery-mk" .. level + 1 })
    end
    if config.chemical_plants then
        table.insert(effects, { type = "unlock-recipe", recipe = "chemical-plant-mk" .. level + 1 })
    end
    
    return effects
    
end

local function get_ingredients_for_level(level)
    
    local blue = 
    {
        {"science-pack-1", 1},
        {"science-pack-2", 1},
        {"science-pack-3", 1},
    }
    
    if level <= 2 then
        return blue
    else
        table.insert(blue, {"alien-science-pack", 1})
        return blue
    end
    
end

local function get_prequisites_for_level(level)
    
    local preqs = {}
    
    if level == 1 then
        preqs = { "advanced-oil-processing", "fluid-handling", "speed-module" }
    elseif level == 2 then
        preqs = { "speed-module-2", "effectivity-module-2" }
    elseif level == 3 then
        preqs = { "alien-technology", "speed-module-3" }
    end
    
    if level == 2 then
        table.insert(preqs, 1, "compact-oil-processing" )
    elseif level >= 3 then
        table.insert(preqs, 1, "compact-oil-processing-" .. level - 1 )
    end
    
    return preqs
    
end

local function make_compact_tech_for_level(level)
    
    return
    {
        type = "technology",
        name = level == 1 and "compact-oil-processing" or "compact-oil-processing-" .. level,
        icon = "__Compact_Factory__/graphics/technology/compact-oil-processing.png",
        icon_size = 128,
        effects = get_effects_for_level(level),
        prerequisites = get_prequisites_for_level(level),
        upgrade = true,
        unit =
        {
            count = (100 * math.pow(level, 2)) + 100,
            ingredients = get_ingredients_for_level(level),
            time = 15 * level,
        },
        order = "d-b-a",
    }
    
end

local CompactTech = {}

if config.refineries or config.chemical_plants or config.preserve_techs then
    for level = 1, refinery_chemplant_tiers - 1 do
        table.insert(CompactTech, make_compact_tech_for_level(level))
    end
end

data_extend(CompactTech)
