
local function get_effects_for_level(level)
    
    local effects = {}
    
    if config.boilers then
        table.insert(effects, { type = "unlock-recipe", recipe = "boiler-mk" .. level + 1 })
    end
    if config.steam_engines then
        table.insert(effects, { type = "unlock-recipe", recipe = "steam-engine-mk" .. level + 1 })
    end
    
    return effects
    
end

local function get_ingredients_for_level(level)
    
    local blue = 
    {
        {"science-pack-1", 1},
        {"science-pack-2", 1},
        {"science-pack-3", 1},
    }
    
    if level <= 2 then
        return blue
    else
        table.insert(blue, {"alien-science-pack", 1})
        return blue
    end
    
end

local function get_prequisites_for_level(level)
    
    local preqs = {}
    
    if level == 1 then
        preqs = { "steel-processing", "fluid-handling", "speed-module", "effectivity-module" }
    elseif level == 2 then
        preqs = { "speed-module-2", "effectivity-module-2" }
    elseif level == 3 then
        preqs = { "alien-technology", "speed-module-3", "effectivity-module-3" }
    end
    
    if level == 2 then
        table.insert(preqs, 1, "compact-steam-energy" )
    elseif level >= 3 then
        table.insert(preqs, 1, "compact-steam-energy-" .. level - 1 )
    end
    
    return preqs
    
end

local function make_compact_tech_for_level(level)
    
    return
    {
        type = "technology",
        name = level == 1 and "compact-steam-energy" or "compact-steam-energy-" .. level,
        icon = "__Compact_Factory__/graphics/technology/compact-steam-energy.png",
        icon_size = 128,
        effects = get_effects_for_level(level),
        prerequisites = get_prequisites_for_level(level),
        upgrade = true,
        unit =
        {
            count = (100 * math.pow(level, 2)) + 100,
            ingredients = get_ingredients_for_level(level),
            time = 15 * level,
        },
        order = "d-b-b",
    }
    
end

local CompactTech = {}

if config.boilers or config.steam_engines or config.preserve_techs then
    for level = 1, boiler_engine_tiers - 1 do
        table.insert(CompactTech, make_compact_tech_for_level(level))
    end
end

data_extend(CompactTech)
